from crop_image.update_image import UpdateImage
import adb_tool_py as adb_tool
import adb_tool_py.command as command


class UpdateImageAdb(UpdateImage):
    def __init__(self, name: str) -> None:
        super().__init__(self, name)

    def update(self, path:str) -> bool:
        adb = adb_tool.AdbTool(serial=self.get_name())
        adb.capture_screenshot()
        adb.save_screenshot(path)
        return True

result = command.command('adb', 'devices')
lines = result.stdout.splitlines()
devices = [line for line in lines if line.strip() and not line.startswith('List of devices attached')]
for device in devices:
    UpdateImageAdb(device.split()[0])



from abc import ABC, abstractmethod


class UpdateImage(ABC):
    s_update_func_list = []

    def __init__(self, update_iamge: 'UpdateImage', name: str) -> None:
        UpdateImage.s_update_func_list.append(update_iamge)
        self.name = name

    @staticmethod
    def get_update_func(name: str) -> 'UpdateImage':
        for func in UpdateImage.s_update_func_list:
            if func.get_name() == name:
                return func
        return None

    def get_name(self) -> str:
        return self.name

    @abstractmethod
    def update(self, path:str) -> bool:
        pass

    def __str__(self) -> str:
        return self.name



from crop_image.update_image import UpdateImage
import platform


class UpdateImageDesktop(UpdateImage):
    def __init__(self) -> None:
        super().__init__(self, 'Desktop')

    def update(self, path: str) -> bool:
        screenshot = self.take_screenshot()
        screenshot.save(path, format="PNG")
        return True

    def take_screenshot(self):
        system = platform.system()
        if system == "Windows":
            return self.take_screenshot_windows()
        elif system == "Darwin":
            return self.take_screenshot_mac()
        elif system == "Linux":
            return self.take_screenshot_linux()
        else:
            raise NotImplementedError(f"Unsupported OS: {system}")

    def take_screenshot_windows(self):
        import pyautogui
        screenshot = pyautogui.screenshot()
        return screenshot

    def take_screenshot_mac(self):
        from PIL import ImageGrab
        screenshot = ImageGrab.grab()
        return screenshot

    def take_screenshot_linux(self):
        import pyscreenshot as ImageGrab
        screenshot = ImageGrab.grab()
        return screenshot


UpdateImageDesktop()



from setuptools import setup, find_packages

setup(
    name='cropimg',
    version='0.1.2',
    packages=find_packages(),
    install_requires=[
        'Flask',
        'pillow',
        'adb-tool-py>=0.1.2',
    ],
    entry_points={
        'console_scripts': [
            'cropimg=crop_image.cropimg:main',
        ],
    },
    author='Shota Iuchi',
    author_email='shotaiuchi.develop@gmail.com',
    description='Cropping images',
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    keywords='crop, image, adb, screenshot',
    url='https://github.com/ShotaIuchi/crop-image',
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime
import io
import os
import base64
from flask import Flask, request, send_file, render_template, abort, jsonify
from PIL import Image
from crop_image.update_image import UpdateImage
from crop_image.update_image_adb import *
from crop_image.update_image_desktop import *

# Input image file path
IMAGE_PATH = os.path.join(os.path.dirname(__file__), 'static', 'input.png')

# Output image directory
OUTPUT_DIR = os.path.join(os.getcwd(), 'output')

# Flask engine
app = Flask(__name__, template_folder='.')

# Update image object name
g_update_image_name: str = UpdateImage.s_update_func_list[0].get_name()


@app.route('/')
def home():
    return render_template('index.html', image_file=IMAGE_PATH)


@app.route('/crop', methods=['POST'])
def crop():
    # request parameters
    startX = request.form.get('startX', type=int, default=0)
    startY = request.form.get('startY', type=int, default=0)
    endX = request.form.get('endX', type=int, default=0)
    endY = request.form.get('endY', type=int, default=0)
    filename = request.form.get('filename', default=f'{datetime.now().strftime("%Y%m%dT%H%M%S")}.png')

    # Input validation
    if not filename.strip():
        return "File name is required.", 400

    # Normalize coordinates
    if startX > endX:
        startX, endX = endX, startX
    if startY > endY:
        startY, endY = endY, startY

    if startX == endX or startY == endY:
        return "Invalid rectangle coordinates.", 400

    # ensure filename has .png extension
    if not filename.lower().endswith('.png'):
        filename += '.png'

    # load image
    img_path = os.path.join(app.root_path, 'static', IMAGE_PATH)
    if not os.path.exists(img_path):
        abort(404, description="Image not found")

    # open image
    img = Image.open(img_path)

    # crop image
    cropped_img = img.crop((startX, startY, endX, endY))

    # save image to output directory
    os.makedirs(os.path.join(app.root_path, OUTPUT_DIR), exist_ok=True)
    cropped_img.save(os.path.join(app.root_path, OUTPUT_DIR, filename))

    # response image to client
    img_io = io.BytesIO()
    cropped_img.save(img_io, 'PNG')
    img_io.seek(0)
    return send_file(img_io, mimetype='image/png')


@app.route('/image_list', methods=['GET'])
def image_list():
    images: list = []
    for ui in UpdateImage.s_update_func_list:
        images.append(ui.get_name())
    return jsonify({"images": images})


@app.route('/get_image', methods=['GET'])
def get_image():
    if os.path.exists(IMAGE_PATH):
        with open(IMAGE_PATH, "rb") as image_file:
            encoded_image = base64.b64encode(image_file.read()).decode('utf-8')
        return jsonify({"image": encoded_image})
    else:
        return jsonify({"error": "Image not found"}), 404


@app.route('/update_image', methods=['POST'])
def update_image():
    global g_update_image_name
    data = request.json
    image_src = data.get('image', g_update_image_name)
    update_image: UpdateImage = None
    for ui in UpdateImage.s_update_func_list:
        if ui.get_name() == image_src:
            update_image = ui
            break
    try:
        update_image.update(IMAGE_PATH)
        return jsonify(success=True)
    except Exception as e:
        return jsonify(success=False, error=str(e))


def main():
    app.run(debug=True)


if __name__ == '__main__':
    main()
